@@ -153,6 +153,7 @@ class LoginHandler(IPythonHandler):
 
     @authenticated
     async def post(self):
+        global domain
         try:
             data = json.loads(self.request.body)
             ## Login
@@ -160,7 +161,7 @@ class LoginHandler(IPythonHandler):
             update_domain(domain)
             loginName = data['loginName']
             password = data['password']
-            login_status, message = await IOLoop.current().run_in_executor(None, login_rest_server, domain, loginName, password)
+            login_status, message = await IOLoop.current().run_in_executor(None, login_rest_server, loginName, password)
             if login_status == 1:
                 ### NOTE: If we succeed logging in but fail after, we want to try to logout
                 ## Exchange versions
@@ -317,8 +318,7 @@ class SetupNotebookHandler(IPythonHandler):
             name = data['name']
             timestamp = data['timestamp']
             notebook = data['notebook']
-            script = data['script']
-            response = await IOLoop.current().run_in_executor(None, setup_notebook, name, timestamp, notebook, script)
+            response = await IOLoop.current().run_in_executor(None, setup_notebook, name, timestamp, notebook)
             self.set_status(401 if response.geturl().endswith('/login') else response.getcode())
             self.write(response.read())
             if response.getcode() >= 300: jupyter_log.info(optumi_start(self) + str(response))
@@ -343,9 +343,8 @@ class LaunchNotebookHandler(IPythonHandler):
             dataFiles = data.get('dataFiles')                   # we use .get() for params that are not required
             compress = data['compress']
             uuid = data['uuid']
-            notebook = data['notebook']
             timestamp = data['timestamp']
-            IOLoop.current().run_in_executor(None, launch_notebook, requirementsFile, dataFiles, compress, uuid, notebook, timestamp)
+            IOLoop.current().run_in_executor(None, launch_notebook, requirementsFile, dataFiles, compress, uuid, timestamp)
             self.write(json.dumps({'message': 'success'}))
         except Exception as e:
             self.set_status(500)
@@ -591,17 +590,17 @@ class PushWorkloadStatusUpdateHandler(IPythonHandler):
             jupyter_log.error(optumi_start(self) + str(e))
             if DEBUG: raise e
 
-class PullWorkloadStatusUpdateHandler(IPythonHandler):
+class PullWorkloadStatusUpdatesHandler(IPythonHandler):
     @authenticated
     async def post(self):
         try:
             data = json.loads(self.request.body)
-            uuid = data['uuid']
-            lastInitializingLine = data['lastInitializingLine']
-            lastUploadingLine = data['lastUploadingLine']
-            lastRequisitioningLine = data['lastRequisitioningLine']
-            lastRunningLine = data['lastRunningLine']
-            response = await IOLoop.current().run_in_executor(None, pull_workload_status_update, uuid, lastInitializingLine, lastUploadingLine, lastRequisitioningLine, lastRunningLine)
+            uuids = data['uuids']
+            lastInitializingLines = data['lastInitializingLines']
+            lastUploadingLines = data['lastUploadingLines']
+            lastRequisitioningLines = data['lastRequisitioningLines']
+            lastRunningLines = data['lastRunningLines']
+            response = await IOLoop.current().run_in_executor(None, pull_workload_status_updates, uuids, lastInitializingLines, lastUploadingLines, lastRequisitioningLines, lastRunningLines)
             self.set_status(401 if response.geturl().endswith('/login') else response.getcode())
             self.write(response.read())
             if response.getcode() >= 300: jupyter_log.info(optumi_start(self) + str(response))
@@ -622,11 +621,11 @@ class PullModuleStatusUpdateHandler(IPythonHandler):
     async def post(self):
         try:
             data = json.loads(self.request.body)
-            workloadUUID = data['workloadUUID']
-            moduleUUID = data['moduleUUID']
-            lastUpdateLine = data['lastUpdateLine']
-            lastOutputLine = data['lastOutputLine']
-            response = await IOLoop.current().run_in_executor(None, pull_module_status_update, workloadUUID, moduleUUID, lastUpdateLine, lastOutputLine)
+            workloadUUIDs = data['workloadUUIDs']
+            moduleUUIDs = data['moduleUUIDs']
+            lastUpdateLines = data['lastUpdateLines']
+            lastOutputLines = data['lastOutputLines']
+            response = await IOLoop.current().run_in_executor(None, pull_module_status_updates, workloadUUIDs, moduleUUIDs, lastUpdateLines, lastOutputLines)
             self.set_status(401 if response.geturl().endswith('/login') else response.getcode())
             self.write(response.read())
             if response.getcode() >= 300: jupyter_log.info(optumi_start(self) + str(response))
@@ -944,8 +943,8 @@ def setup_handlers(server_app):
     web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/add-data-connector'), AddDataConnectorHandler)])
     web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/remove-data-connector'), RemoveDataConnectorHandler)])
     web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/push-workload-status-update'), PushWorkloadStatusUpdateHandler)])
-    web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/pull-workload-status-update'), PullWorkloadStatusUpdateHandler)])
-    web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/pull-module-status-update'), PullModuleStatusUpdateHandler)])
+    web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/pull-workload-status-updates'), PullWorkloadStatusUpdatesHandler)])
+    web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/pull-module-status-updates'), PullModuleStatusUpdateHandler)])
     web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/push-module-input'), PushModuleInputHandler)])
     web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/save-notebook-output-file'), SaveNotebookOutputFileHandler)])
     web_app.add_handlers(host_pattern, [(url_path_join(base_url, '/optumi/get-file-download-progress'), GetFileDownloadProgressHandler)])
@@ -971,7 +970,7 @@ def install_auth_opener(loginName, password):
     info = urlencode({"username": loginName, "password": password}).encode('utf8')
     return info
 
-def login_rest_server(domain, loginName, password):
+def login_rest_server(loginName, password):
     try:
         # Since we are bypassing the hostname check for the SSL context, we manually check it here
         cert = ssl.get_server_certificate((domain, 8443))
@@ -1096,14 +1095,13 @@ def preview_notebook(notebook):
     except HTTPError as e:
         return e
 
-def setup_notebook(name, timestamp, notebook, script):
+def setup_notebook(name, timestamp, notebook):
     URL = get_path() + '/exp/jupyterlab/setup-notebook'
     try:
         form = MultiPartForm()
         form.add_field('name', name)
         form.add_field('timestamp', timestamp)
         form.add_file('notebook', notebook['path'], fileHandle=io.BytesIO(notebook['content'].encode('utf-8')))
-        form.add_field('script', script)
         data = bytes(form)
 
         req = request.Request(URL, data=data)
@@ -1113,13 +1111,12 @@ def setup_notebook(name, timestamp, notebook, script):
     except HTTPError as e:
         return e
 
-def launch_notebook(requirementsFile, dataFiles, compress, uuid, notebook, timestamp):
+def launch_notebook(requirementsFile, dataFiles, compress, uuid, timestamp):
     URL = get_path() + '/exp/jupyterlab/launch-notebook'
     try:
         form = MultiPartForm()
         form.add_field('compressed', str(compress))
         form.add_field('uuid', uuid)
-        form.add_file('notebook', notebook['path'], fileHandle=io.BytesIO(notebook['content'].encode('utf-8')))
         ## Send this list as a file since it can have arbitrary length and cause issues for the REST Interface if sent as an array
         form.add_file('dataFileNames', 'dataFileNames', fileHandle=io.BytesIO(','.join(dataFiles).encode('utf-8')))
         form.add_field('timestamp', timestamp)
@@ -1362,15 +1359,20 @@ def push_workload_status_update(uuid, phase, update):
     except HTTPError as e:
         return e
 
-def pull_workload_status_update(uuid, lastInitializingLine, lastUploadingLine, lastRequisitioningLine, lastRunningLine):
-    URL = get_path() + '/exp/jupyterlab/pull-workload-status-update'
+def pull_workload_status_updates(uuids, lastInitializingLines, lastUploadingLines, lastRequisitioningLines, lastRunningLines):
+    URL = get_path() + '/exp/jupyterlab/pull-workload-status-updates'
     try:
         form = MultiPartForm()
-        form.add_field('uuid', uuid)
-        form.add_field('lastInitializingLine', lastInitializingLine)
-        form.add_field('lastUploadingLine', lastUploadingLine)
-        form.add_field('lastRequisitioningLine', lastRequisitioningLine)
-        form.add_field('lastRunningLine', lastRunningLine)
+        for uuid in uuids:
+            form.add_field('uuids', uuid)
+        for lastInitializingLine in lastInitializingLines:
+            form.add_field('lastInitializingLines', str(lastInitializingLine))
+        for lastUploadingLine in lastUploadingLines:
+            form.add_field('lastUploadingLines', str(lastUploadingLine))
+        for lastRequisitioningLine in lastRequisitioningLines:
+            form.add_field('lastRequisitioningLines', str(lastRequisitioningLine))
+        for lastRunningLine in lastRunningLines:
+            form.add_field('lastRunningLines', str(lastRunningLine))
         data = bytes(form)
 
         req = request.Request(URL, data=data)
@@ -1380,14 +1382,18 @@ def pull_workload_status_update(uuid, lastInitializingLine, lastUploadingLine, l
     except HTTPError as e:
         return e
 
-def pull_module_status_update(workloadUUID, moduleUUID, lastUpdateLine, lastOutputLine):
-    URL = get_path() + '/exp/jupyterlab/pull-module-status-update'
+def pull_module_status_updates(workloadUUIDs, moduleUUIDs, lastUpdateLines, lastOutputLines):
+    URL = get_path() + '/exp/jupyterlab/pull-module-status-updates'
     try:
         form = MultiPartForm()
-        form.add_field('workloadUUID', workloadUUID)
-        form.add_field('moduleUUID', moduleUUID)
-        form.add_field('lastUpdateLine', lastUpdateLine)
-        form.add_field('lastOutputLine', lastOutputLine)
+        for workloadUUID in workloadUUIDs:
+            form.add_field('workloadUUIDs', workloadUUID)
+        for moduleUUID in moduleUUIDs:
+            form.add_field('moduleUUIDs', moduleUUID)
+        for lastUpdateLine in lastUpdateLines:
+            form.add_field('lastUpdateLines', str(lastUpdateLine))
+        for lastOutputLine in lastOutputLines:
+            form.add_field('lastOutputLines', str(lastOutputLine))
         data = bytes(form)
 
         req = request.Request(URL, data=data)
@@ -1720,7 +1726,7 @@ class MultiPartForm:
 DEBUG_SESSIONS = False
 
 def handle_session(module, server):
-    #jupyter_log.info(optumi_start() + 'Session connected for ' + module, flush=True)
+    # jupyter_log.info(optumi_start() + 'Session connected for ' + module)
 
     try:
         inputs = [server]
@@ -1823,4 +1829,4 @@ def handle_session(module, server):
     except Exception as e:
         jupyter_log.warn(optumi_start() + 'Session closed: ' + str(e))
 
-    #jupyter_log.info(optumi_start() + 'Session disconnected for ' + module, flush=True)
+    # jupyter_log.info(optumi_start() + 'Session disconnected for ' + module)
