@@ -32,7 +32,7 @@ export enum StatusColor {
 export class StatusWrapper extends React.Component<IProps, IState> {
 
     public render = (): JSX.Element => {
-        if (Global.shouldLogOnRender) console.log('StatusRender (' + new Date().getSeconds() + ')');
+        if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
 
         return (
             <div style={this.props.style}>
@@ -49,7 +49,7 @@ export class StatusWrapper extends React.Component<IProps, IState> {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
