@@ -51,6 +51,7 @@ interface IState {
 
 // TODO:Beck The popup needs to be abstracted out, there is too much going on to reproduce it in more than one file
 export class UserDialog extends React.Component<IProps, IState> {
+    private _isMounted = false
 
     constructor(props: IProps) {
         super(props);
@@ -63,7 +64,7 @@ export class UserDialog extends React.Component<IProps, IState> {
 
     private resolveOpenToPromise: undefined | ((value: void | PromiseLike<void>) => void) = undefined
     public openTo = (page: Page) => {
-        this.setState({open: true, selectedPanel: page})
+        this.safeSetState({open: true, selectedPanel: page})
         return new Promise<void>(resolve => {
             this.resolveOpenToPromise = resolve
         })
@@ -71,11 +72,11 @@ export class UserDialog extends React.Component<IProps, IState> {
     
     private handleClickOpen = () => {
         if (this.props.onOpen) this.props.onOpen()
-		this.setState({ open: true, selectedPanel: Page.GENERAL });
+		this.safeSetState({ open: true, selectedPanel: Page.GENERAL });
 	}
 
 	private handleClose = () => {
-        this.setState({ open: false });
+        this.safeSetState({ open: false });
         if (this.props.onClose) this.props.onClose();
         if (this.resolveOpenToPromise) {
 			this.resolveOpenToPromise()
@@ -102,7 +103,7 @@ export class UserDialog extends React.Component<IProps, IState> {
 	}
 
     public render = (): JSX.Element => {
-        if (Global.shouldLogOnRender) console.log('UsersDialogRender (' + new Date().getSeconds() + ')');
+        if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         var defaultProfilePicture: string = Global.user.name.replace(/(?<=\B)\w+/g, '').replace(/[ ]/g, '').toUpperCase()
         return (
             <div style={Object.assign({}, this.props.style)} >
@@ -165,13 +166,13 @@ export class UserDialog extends React.Component<IProps, IState> {
                                 cancel={{
                                     text: `Cancel`,
                                     onCancel: (prevent: boolean) => {
-                                        this.setState({ showLogoutWithSessionPopup: false })
+                                        this.safeSetState({ showLogoutWithSessionPopup: false })
                                     },
                                 }}
                                 continue={{
                                     text: `Close it`,
                                     onContinue: (prevent: boolean) => {
-                                        this.setState({ showLogoutWithSessionPopup: false })
+                                        this.safeSetState({ showLogoutWithSessionPopup: false })
                                         for (let app of Global.user.appTracker.activeSessions) {
                                             this.handleStopClicked(app);
                                         }
@@ -188,7 +189,7 @@ export class UserDialog extends React.Component<IProps, IState> {
                                 onClick={() => {
                                     const user: User = Global.user;
                                     if (user.appTracker.activeSessions.length != 0) {
-                                        this.setState({ showLogoutWithSessionPopup: true });
+                                        this.safeSetState({ showLogoutWithSessionPopup: true });
                                     } else {
                                         this.logout()
                                     }
@@ -223,7 +224,7 @@ export class UserDialog extends React.Component<IProps, IState> {
                                 <div style={{padding: '6px'}}>
                                     <Tabs
                                         value={this.state.selectedPanel}
-                                        onChange={(event, newValue) => this.setState({selectedPanel: newValue})}
+                                        onChange={(event, newValue) => this.safeSetState({selectedPanel: newValue})}
                                         orientation='vertical'
                                         variant='fullWidth'
                                         indicatorColor='primary'
@@ -350,10 +351,34 @@ export class UserDialog extends React.Component<IProps, IState> {
 	}
 
     public componentDidMount = () => {
+        this._isMounted = true
         if (this.props.getOpenTo) this.props.getOpenTo(this.openTo);
     }
 
     public componentWillUnmount = () => {
         if (this.props.getOpenTo) this.props.getOpenTo(() => new Promise<void>(() => {}));
+        this._isMounted = false
     }
+
+    private safeSetState = (map: any) => {
+		if (this._isMounted) {
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
+		}
+	}
 }
