@@ -15,6 +15,7 @@ import {
 	List,
 } from '@material-ui/core';
 import { getStyledSwitch } from '../../core/Switch';
+import ExtraInfo from '../../utils/ExtraInfo';
 
 interface IProps {
 	app: App;
@@ -71,15 +72,16 @@ export class OutputFileList extends React.Component<IProps, IState> {
 				<div>
                     <div style={{display: 'inline-flex', alignItems: 'center'}}>
                         <div style={{paddingLeft: '16px', paddingRight: '16px'}}>
-                            <StyledSwitch
-                                title='Overwrite existing files with downloaded files, or rename the downloaded files'
-                                color='primary'
-                                inputProps={{style: {height: '24px'}}}
-                                checked={this.state.overwrite}
-                                onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
-                                    this.checkAndSetState({overwrite: event.currentTarget.checked})
-                                }}
-                            />
+                            <ExtraInfo reminder='Overwrite existing files with downloaded files, or rename the downloaded files'>
+								<StyledSwitch
+									color='primary'
+									inputProps={{style: {height: '24px'}}}
+									checked={this.state.overwrite}
+									onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
+										this.safeSetState({overwrite: event.currentTarget.checked})
+									}}
+								/>
+							</ExtraInfo>
                        </div>
                         Overwrite existing files
                     </div>
@@ -135,8 +137,8 @@ export class OutputFileList extends React.Component<IProps, IState> {
 		}
 	}
 
-	render() {
-		if (Global.shouldLogOnRender) console.log('OutputFileListRender (' + new Date().getSeconds() + ')');
+	public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
 		return (
 			<div style={{padding: '12px', width: "100%"}}>
 				{this.getFiles()}
@@ -145,18 +147,34 @@ export class OutputFileList extends React.Component<IProps, IState> {
 	}
 
 	// Will be called automatically when the component is mounted
-	componentDidMount = () => {
+	public componentDidMount = () => {
 		this._isMounted = true;
 	}
 
 	// Will be called automatically when the component is unmounted
-	componentWillUnmount = () => {
+	public componentWillUnmount = () => {
 		this._isMounted = false;
 	}
 
-	private checkAndSetState = (map: any) => {
+	private safeSetState = (map: any) => {
 		if (this._isMounted) {
-			this.setState(map);
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
 		}
 	}
 
@@ -164,7 +182,7 @@ export class OutputFileList extends React.Component<IProps, IState> {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
