@@ -37,7 +37,10 @@ export class PreviewLaunchButton extends React.Component<IProps, IState> {
 
 	private poll = () => {
 		// We will poll for a new preview every 10 seconds
-		if (this.polling) setTimeout(() => this.poll(), 10000);
+		if (this.polling) {
+			if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+			setTimeout(() => this.poll(), 10000);
+		}
 		this.handlePreviewClick(false);
 	}
 
@@ -50,9 +53,9 @@ export class PreviewLaunchButton extends React.Component<IProps, IState> {
             // Make sure the notebook has the correct metadata.
 			if (bypassLimiter || !retrievingPreview) {
 				retrievingPreview = true;
-				const app = new App(current.context.path, current.model.toJSON(), "");
+				const app = new App(current.context.path, current.model.toJSON());
 				app.previewNotebook(printRecommendations).then((machines: Machine[]) => {
-					this.checkAndSetState({
+					this.safeSetState({
 						machine: machines,
 					});
 					if (updatePreviewAgain) {
@@ -100,15 +103,15 @@ export class PreviewLaunchButton extends React.Component<IProps, IState> {
 	// 	if (current != null) {
 	// 		const app = new App(current.context.path, current.model.toJSON(), "");
 	// 		app.previewNotebook(printRecommendations).then((machine: Machine) => {
-	// 			this.checkAndSetState({
+	// 			this.safeSetState({
 	// 				machine: machine,
 	// 			});
 	// 		});
 	// 	}
 	// }
 
-	public render(): JSX.Element {
-        if (Global.shouldLogOnRender) console.log('PreviewLaunchButtonRender (' + new Date().getSeconds() + ')');
+	public render = (): JSX.Element => {
+        if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         var order = 1;
 		return (
 			<div style={Object.assign({width: '100%'}, this.props.style)}>
@@ -136,9 +139,25 @@ export class PreviewLaunchButton extends React.Component<IProps, IState> {
 		this.polling = false;
 	}
 
-	private checkAndSetState = (map: any) => {
+	private safeSetState = (map: any) => {
 		if (this._isMounted) {
-			this.setState(map);
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
 		}
 	}
 
@@ -146,7 +165,7 @@ export class PreviewLaunchButton extends React.Component<IProps, IState> {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
