@@ -76,7 +76,7 @@ export default class FileBrowser extends React.Component<IProps, IState> {
                 const depth = file.path.replace(/[^/]/g, '').length
                 while (newPath.length > depth) newPath.pop();
                 if (file !== this.state.root) newPath.push(file)
-                this.setState({
+                this.safeSetState({
                     path: newPath,
                     files: json.content,
                 })
@@ -87,6 +87,7 @@ export default class FileBrowser extends React.Component<IProps, IState> {
     }
 
     public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         return (
             <div className='jp-FileBrowser' style={this.props.style}>
                 <BreadCrumbs serverRoot={this.state.serverRoot} root={this.state.root} path={this.state.path} onOpen={this.handleOpen} />
@@ -99,10 +100,10 @@ export default class FileBrowser extends React.Component<IProps, IState> {
         )
     }
 
-    componentDidMount = () => {
+    public componentDidMount = () => {
         this._isMounted = true
         this.request('/contents').then(json => {
-            if (this._isMounted) this.setState({root: json, files: json.content});
+            this.safeSetState({root: json, files: json.content});
         })
         // Override the JupyterLab context menu open (disable it)
         this.oldOpen = Global.lab.contextMenu.open;
@@ -110,11 +111,33 @@ export default class FileBrowser extends React.Component<IProps, IState> {
     }
 
     // Add context menu items back
-    componentWillUnmount = () => {
+    public componentWillUnmount = () => {
         // Restore the old JupyterLab context menu open
         Global.lab.contextMenu.open = this.oldOpen;
         this._isMounted = false
     }
+
+    private safeSetState = (map: any) => {
+		if (this._isMounted) {
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
+		}
+	}
 }
 
 
