@@ -54,6 +54,7 @@ export class AccountGeneralSubMenu extends React.Component<IProps, IAccountGener
     private saveCompressFilesEnabledValue(compressFilesEnabled: boolean) { Global.user.compressFilesEnabled = compressFilesEnabled }
 
     public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         return (
             <>
                 <div style={Object.assign({padding: '6px'}, this.props.style)}>
@@ -97,7 +98,7 @@ export class AccountGeneralSubMenu extends React.Component<IProps, IAccountGener
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
@@ -112,6 +113,7 @@ interface IAccountLimitsSubMenuState {
 }
 
 export class AccountLimitsSubMenu extends React.Component<IProps, IAccountLimitsSubMenuState> {
+    private _isMounted = false
 
     constructor(props: IProps) {
         super(props);
@@ -137,6 +139,7 @@ export class AccountLimitsSubMenu extends React.Component<IProps, IAccountLimits
     private saveUserHoldoverTimeValue(userHoldoverTime: number) { Global.user.userHoldoverTime = userHoldoverTime }
 
     public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         return (
             <>
                 <div style={Object.assign({padding: '6px'}, this.props.style)}>
@@ -147,8 +150,8 @@ export class AccountLimitsSubMenu extends React.Component<IProps, IAccountLimits
                         unstyleUnitValue={(value: string) => { return value.replace('$', '').replace('.', '').replace(/\d/g, '').length > 0 ? Number.NaN : Number.parseFloat(value.replace('$', '')) }}
                         label='Budget'
                         labelWidth={LABEL_WIDTH}
-                        onFocus={() => this.setState({budgetFocused: true})}
-                        onBlur={() => this.setState({budgetFocused: false})}
+                        onFocus={() => this.safeSetState({budgetFocused: true})}
+                        onBlur={() => this.safeSetState({budgetFocused: false})}
                         helperText={this.state.budgetFocused ? `Must be between $1 and $${Global.user.maxBudget}` : 'Max monthly spend'}
                         minValue={1}
                         maxValue={Global.user.maxBudget}
@@ -159,8 +162,8 @@ export class AccountLimitsSubMenu extends React.Component<IProps, IAccountLimits
                         saveValue={this.saveUserRecommendationsValue}
                         label='Alternatives'
                         labelWidth={LABEL_WIDTH}
-                        onFocus={() => this.setState({recsFocused: true})}
-                        onBlur={() => this.setState({recsFocused: false})}
+                        onFocus={() => this.safeSetState({recsFocused: true})}
+                        onBlur={() => this.safeSetState({recsFocused: false})}
                         helperText={this.state.recsFocused ? `Must be between 1 and ${Global.user.maxRecommendations}` : 'Number of machines considered'}
                         minValue={1}
                         maxValue={Global.user.maxRecommendations}
@@ -194,8 +197,8 @@ export class AccountLimitsSubMenu extends React.Component<IProps, IAccountLimits
                         ]}
                         label='Holdover'
                         labelWidth={LABEL_WIDTH}
-                        onFocus={() => this.setState({holdoverFocused: true})}
-                        onBlur={() => this.setState({holdoverFocused: false})}
+                        onFocus={() => this.safeSetState({holdoverFocused: true})}
+                        onBlur={() => this.safeSetState({holdoverFocused: false})}
                         helperText={this.state.holdoverFocused ? `Must be between 0 seconds and ${Global.user.maxHoldoverTime / 3600} hours` : 'Time before releasing idle machines'}
                         minValue={0}
                         maxValue={Global.user.maxHoldoverTime}
@@ -205,11 +208,41 @@ export class AccountLimitsSubMenu extends React.Component<IProps, IAccountLimits
         )
     }
 
+    public componentDidMount = () => {
+        this._isMounted = true
+    }
+
+    public componentWillUnmount = () => {
+        this._isMounted = false
+    }
+
+    private safeSetState = (map: any) => {
+		if (this._isMounted) {
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
+		}
+	}
+
     public shouldComponentUpdate = (nextProps: IProps, nextState: IState): boolean => {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
@@ -265,6 +298,7 @@ export class AccountPaymentSubMenu extends React.Component<IProps, IAccountPayme
     }
 
     public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         return (
             <>
                 <div style={Object.assign({padding: '6px'}, this.props.style)}>
@@ -318,13 +352,14 @@ export class AccountPaymentSubMenu extends React.Component<IProps, IAccountPayme
 		).then((response: Response) => {
 			if (this.polling) {
 				// If we are polling, send a new request in 2 seconds
+                if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
 				setTimeout(() => this.receiveUpdate(), 2000);
 			}
 			Global.handleResponse(response);
 			return response.json();
 		}).then((body: any) => {
 			if (body) {
-                this.checkAndSetState({ balance: body.balance });
+                this.safeSetState({ balance: body.balance });
 			}
         });
     }
@@ -350,7 +385,7 @@ export class AccountPaymentSubMenu extends React.Component<IProps, IAccountPayme
 			return response.json();
 		}).then((body: any) => {
 			if (body) {
-                this.checkAndSetState({ billing: body.records });
+                this.safeSetState({ billing: body.records });
                 if (save) {
                     const data: string[][] = [];
                     var sorted: any[] = body.records.sort((n1: any, n2: any) => {
@@ -399,7 +434,7 @@ export class AccountPaymentSubMenu extends React.Component<IProps, IAccountPayme
     }
 
     // Will be called automatically when the component is mounted
-	componentDidMount = () => {
+	public componentDidMount = () => {
         this._isMounted = true;
         this.getDetailedBilling(false);
         this.polling = true;
@@ -407,22 +442,38 @@ export class AccountPaymentSubMenu extends React.Component<IProps, IAccountPayme
 	}
 
 	// Will be called automatically when the component is unmounted
-	componentWillUnmount = () => {
+	public componentWillUnmount = () => {
         this._isMounted = false;
         this.polling = false;
     }
     
-    private checkAndSetState = (map: any) => {
+    private safeSetState = (map: any) => {
 		if (this._isMounted) {
-			this.setState(map);
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
 		}
-    }
+	}
 
     public shouldComponentUpdate = (nextProps: IProps, nextState: IState): boolean => {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
@@ -448,7 +499,7 @@ export class AccountConnectorsSubMenu extends React.Component<IProps, IAccountCo
 
     // Use a key to force the data connector browser to refresh
     private forceNewBrowser = () => {
-        this.setState({ browserKey: this.state.browserKey + 1 })
+        this.safeSetState({ browserKey: this.state.browserKey + 1 })
     }
 
     public request = async () => {
@@ -461,6 +512,7 @@ export class AccountConnectorsSubMenu extends React.Component<IProps, IAccountCo
     }
 
     public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         return (
             <>
                 <div style={Object.assign({}, this.props.style)}>
@@ -491,7 +543,7 @@ export class AccountConnectorsSubMenu extends React.Component<IProps, IAccountCo
                             }).then(() => {
                                 var newDataConnectors = this.state.dataConnectors
                                 newDataConnectors = newDataConnectors.filter(dataConnector => dataConnector.name !== dataConnectorMetadata.name)
-                                this.setState({dataConnectors: newDataConnectors})
+                                this.safeSetState({dataConnectors: newDataConnectors})
                                 this.forceNewBrowser()
                             });                            
                         }}
@@ -502,11 +554,11 @@ export class AccountConnectorsSubMenu extends React.Component<IProps, IAccountCo
                     </div>
                     <Divider />
                     <div style={{marginBottom: '6px'}} />
-                    <AmazonS3ConnectorPopup handleClose={this.forceNewBrowser} />
-                    <GoogleCloudStorageConnectorPopup handleClose={this.forceNewBrowser} />
-                    <GoogleDriveConnectorPopup handleClose={this.forceNewBrowser} />
-                    <KaggleConnectorPopup handleClose={this.forceNewBrowser} />
-                    <WasabiConnectorPopup handleClose={this.forceNewBrowser} />
+                    <AmazonS3ConnectorPopup onClose={this.forceNewBrowser} />
+                    <GoogleCloudStorageConnectorPopup onClose={this.forceNewBrowser} />
+                    <GoogleDriveConnectorPopup onClose={this.forceNewBrowser} />
+                    <KaggleConnectorPopup onClose={this.forceNewBrowser} />
+                    <WasabiConnectorPopup onClose={this.forceNewBrowser} />
                 </div>
             </>
         )
@@ -514,20 +566,40 @@ export class AccountConnectorsSubMenu extends React.Component<IProps, IAccountCo
 
     public componentDidMount = () => {
         this._isMounted = true
-        this.request().then(json => {
-            if (this._isMounted) this.setState({dataConnectors: json.connectors});
-        })
+        this.request().then(json => this.safeSetState({dataConnectors: json.connectors}))
     }
 
     public componentWillUnmount = () => {
         this._isMounted = false
     }
 
+    private safeSetState = (map: any) => {
+		if (this._isMounted) {
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
+		}
+	}
+
     public shouldComponentUpdate = (nextProps: IProps, nextState: IAccountConnectorsSubMenuState): boolean => {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
@@ -537,6 +609,7 @@ export class AccountConnectorsSubMenu extends React.Component<IProps, IAccountCo
 
 export class TeamSubMenu extends React.Component<IProps, IState> {
     public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         return (
             <Header title='Members' />
         )
