@@ -14,15 +14,17 @@ import DoneIcon from '@material-ui/icons/Done';
 import VisibilityIcon from '@material-ui/icons/Visibility';
 import VisibilityOffIcon from '@material-ui/icons/VisibilityOff';
 import LockIcon from '@material-ui/icons/Lock';
+import ExtraInfo from '../utils/ExtraInfo';
 
 interface IProps<T> {
     style?: CSSProperties
-    label: string
+    label?: string
     labelWidth?: string
     getValue: () => T
     saveValue: (value: T) => string | void
     editPressRequired?: boolean
     editPopup?: JSX.Element
+    disabled?: boolean
     disabledMessage?: string
     password?: boolean
     placeholder?: string
@@ -34,6 +36,7 @@ interface IProps<T> {
     onFocus?: (event: React.FocusEvent<HTMLInputElement>) => void
     onBlur?: (event: React.FocusEvent<HTMLInputElement>) => void
     multiline?: boolean
+    minLines?: number
     required?: boolean
 }
 
@@ -47,6 +50,8 @@ interface IState<T> {
 }
 
 export class TextBox<T> extends React.Component<IProps<T>, IState<T>> {
+    private _isMounted = false
+
     StyledOutlinedInput: any
     StyledInputLabel: any;
     textField: React.RefObject<HTMLInputElement>
@@ -111,43 +116,43 @@ export class TextBox<T> extends React.Component<IProps<T>, IState<T>> {
     // This turns the textbox red if it is not a valid value determined by the unstyle function
     private handleChange = async (event: React.FormEvent<HTMLInputElement>) => {
         const value = event.currentTarget.value
-        this.setState({ textValue: value });
+        this.safeSetState({ textValue: value });
         if (this.props.unstyleUnitValue == undefined) {
             if (typeof value === 'number') {
                 if (this.props.minValue !== undefined && value < this.props.minValue) {
-                    this.setState({ invalidTextMessage: 'Minimum value is ' + this.props.styledUnitValue(this.props.minValue) })
+                    this.safeSetState({ invalidTextMessage: 'Minimum value is ' + this.props.styledUnitValue(this.props.minValue) })
                 } else if (this.props.maxValue !== undefined && value > this.props.maxValue) {
-                    this.setState({ invalidTextMessage: 'Maximum value is ' + this.props.styledUnitValue(this.props.maxValue) })
+                    this.safeSetState({ invalidTextMessage: 'Maximum value is ' + this.props.styledUnitValue(this.props.maxValue) })
                 } else {
-                    if (!this.props.editPressRequired) this.setState({ value: value as unknown as T, invalidTextMessage: '' });
+                    if (!this.props.editPressRequired) this.safeSetState({ value: value as unknown as T, invalidTextMessage: '' });
                 }
             } else {
-                if (!this.props.editPressRequired) this.setState({ value: value as unknown as T, invalidTextMessage: '' });
+                if (!this.props.editPressRequired) this.safeSetState({ value: value as unknown as T, invalidTextMessage: '' });
             }
         } else {
             var unstyledValue: any = this.props.unstyleUnitValue(value);
             if (!isNaN(unstyledValue)) {
                 if (this.props.minValue !== undefined && unstyledValue < this.props.minValue) {
-                    this.setState({ invalidTextMessage: 'Minimum value is ' + this.props.styledUnitValue(this.props.minValue) })
+                    this.safeSetState({ invalidTextMessage: 'Minimum value is ' + this.props.styledUnitValue(this.props.minValue) })
                 } else if (this.props.maxValue !== undefined && unstyledValue > this.props.maxValue) {
-                    this.setState({ invalidTextMessage: 'Maximum value is ' + this.props.styledUnitValue(this.props.maxValue) })
+                    this.safeSetState({ invalidTextMessage: 'Maximum value is ' + this.props.styledUnitValue(this.props.maxValue) })
                 } else {
-                    if (!this.props.editPressRequired) this.setState({ value: unstyledValue as unknown as T, invalidTextMessage: '' });
+                    if (!this.props.editPressRequired) this.safeSetState({ value: unstyledValue as unknown as T, invalidTextMessage: '' });
                 }
             } else {
-                this.setState({ invalidTextMessage: 'Invalid format' })
+                this.safeSetState({ invalidTextMessage: 'Invalid format' })
             }
         }
     }
 
     private startEdit = () => {
-        this.setState({editing: true})
+        this.safeSetState({editing: true})
         this.textField.current.disabled = false
         this.textField.current.focus()
     }
 
     private discardChanges = () => {
-        this.setState({
+        this.safeSetState({
             editing: false,
             textValue: this.props.styledUnitValue(this.state.value),
             invalidTextMessage: '',
@@ -168,20 +173,20 @@ export class TextBox<T> extends React.Component<IProps<T>, IState<T>> {
                 invalidTextMessage = 'Maximum value is ' + this.props.styledUnitValue(this.props.maxValue)
             }
         }
-        this.setState({invalidTextMessage: invalidTextMessage});
+        this.safeSetState({invalidTextMessage: invalidTextMessage});
         if (invalidTextMessage == '') {
-            this.setState({editing: false, value: unstyledValue, textValue: this.props.styledUnitValue(unstyledValue)})
+            this.safeSetState({editing: false, value: unstyledValue, textValue: this.props.styledUnitValue(unstyledValue)})
             var saveReturn = this.props.saveValue(unstyledValue)
-            this.setState({invalidTextMessage: saveReturn == undefined ? '' : saveReturn as string})
+            this.safeSetState({invalidTextMessage: saveReturn == undefined ? '' : saveReturn as string})
         }
     }
 
     private startHover = () => {
-        this.setState({hovered: true})
+        this.safeSetState({hovered: true})
     }
 
     private stopHover = () => {
-        this.setState({hovered: false})
+        this.safeSetState({hovered: false})
     }
 
     private renderPopupButton = (render: boolean): JSX.Element => {
@@ -190,10 +195,10 @@ export class TextBox<T> extends React.Component<IProps<T>, IState<T>> {
                 display: render ? 'flex' : 'none',
             },
             onOpen: () => {
-                this.setState({editing: true})
+                this.safeSetState({editing: true})
             },
             onClose: () => {
-                this.setState({hovered: false, editing: false})
+                this.safeSetState({hovered: false, editing: false})
             }
         }) : <></>
     }
@@ -285,7 +290,7 @@ export class TextBox<T> extends React.Component<IProps<T>, IState<T>> {
     }
     
     public render = (): JSX.Element => {
-        if (Global.shouldLogOnRender) console.log('ResourceTextBoxRender (' + new Date().getSeconds() + ')');
+        if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         var showPopupButton = false, showEditButton = false, showConfirmCancelButtons = false, showLabel = false
         // There should never be an if without an else. That could lead to nothing being rendered to the left of the textbox
         if (this.props.editPopup === undefined && !this.props.editPressRequired) {
@@ -309,91 +314,128 @@ export class TextBox<T> extends React.Component<IProps<T>, IState<T>> {
                 onMouseOver={this.startHover}
                 onMouseOut={this.stopHover}
             >
-                <div style={{
-                    display: 'flex',
-                    position: 'relative',
-                    minWidth: this.props.labelWidth || '68px',
-                    maxWidth: this.props.labelWidth || '68px',
-                    height: '24px',
-                    margin: '0px 12px',
-                    lineHeight: '24px',
-                    textAlign: 'center',
-                }}>
-                    {this.renderLabel(showLabel)}
-                    {this.renderPopupButton(showPopupButton)}
-                    {this.renderEditButton(showEditButton)}
-                    {this.renderConfirmCancelButtons(showConfirmCancelButtons)}
-                    {this.renderLabelOverButton(!showLabel)}
-                </div>
-                <FormControl
-                    title={this.state.invalidTextMessage}
-                    error={this.state.invalidTextMessage != ''}
-                    variant='outlined'
-                    style={{width: '100%', margin: '2px 6px', height: this.props.multiline ? '' : (this.props.helperText ? '32px' : '20px')}}
-                >
-                    <this.StyledOutlinedInput
-                        inputRef={this.textField}
-                        value={this.state.textValue}
-                        placeholder={this.props.placeholder}
-                        disabled={
-                            !(this.props.disabledMessage == undefined || this.props.disabledMessage == '') ||
-                            (this.props.editPressRequired && !this.state.editing) ||
-                            this.props.editPopup !== undefined
+                {this.props.label && (
+                    <div style={{
+                        display: 'flex',
+                        position: 'relative',
+                        minWidth: this.props.labelWidth || '68px',
+                        maxWidth: this.props.labelWidth || '68px',
+                        height: '24px',
+                        margin: '0px 12px',
+                        lineHeight: '24px',
+                        textAlign: 'center',
+                    }}>
+                        {this.renderLabel(showLabel)}
+                        {this.renderPopupButton(showPopupButton)}
+                        {this.renderEditButton(showEditButton)}
+                        {this.renderConfirmCancelButtons(showConfirmCancelButtons)}
+                        {this.renderLabelOverButton(!showLabel)}
+                    </div>
+                )}
+                {/* <ExtraInfo reminder={this.state.invalidTextMessage}> */}
+                    <FormControl
+                        error={this.state.invalidTextMessage != ''}
+                        variant='outlined'
+                        style={{width: '100%', margin: '2px 6px', height: this.props.multiline ? '' : (this.props.helperText ? '32px' : '20px')}}
+                    >
+                        <this.StyledOutlinedInput
+                            inputRef={this.textField}
+                            value={this.state.textValue}
+                            placeholder={this.props.placeholder}
+                            disabled={
+                                this.props.disabled ||
+                                (!(this.props.disabledMessage == undefined || this.props.disabledMessage == '') ||
+                                (this.props.editPressRequired && !this.state.editing) ||
+                                this.props.editPopup !== undefined)
+                            }
+                            onChange={this.handleChange}
+                            onKeyDown={(event: React.KeyboardEvent) => { if ((!this.props.multiline && event.key == 'Enter') || event.key == 'Escape') this.textField.current.blur() }}
+                            onFocus={this.handleFocus}
+                            onBlur={this.handleBlur}
+                            type={this.props.password && this.state.hidePassword ? 'password' : 'text'}
+                            multiline={this.props.multiline}
+                            required={this.props.required}
+                            rows={(() => {
+                                const lines = this.state.textValue.split('\n').length;
+                                if (this.props.minLines && lines < this.props.minLines) return this.props.minLines;
+                                return lines;
+                            })()}
+                            endAdornment={
+                                (this.props.disabled || !(this.props.disabledMessage == undefined || this.props.disabledMessage == '')) ? (
+                                    <ExtraInfo reminder={this.props.disabledMessage}>
+                                        <InputAdornment position="end" style={{height: '20px', margin: '0px 3px 0px 0px'}}>
+                                            <IconButton
+                                                disabled
+                                                style={{padding: '3px 3px 3px 0px'}}
+                                            >
+                                                <LockIcon style={{width: '14px', height: '14px'}} />
+                                            </IconButton>
+                                        </InputAdornment>
+                                    </ExtraInfo>
+                                ) : this.props.password && (
+                                    <ExtraInfo reminder={this.state.hidePassword ? 'Show Password' : 'Hide Password'}>
+<                                        InputAdornment position="end" style={{height: '20px', margin: '0px 3px 0px 0px'}}>
+                                            <IconButton
+                                                onClick={() => this.safeSetState({hidePassword: !this.state.hidePassword})}
+                                                style={{padding: '3px 3px 3px 0px'}}
+                                            >
+                                                {this.state.hidePassword ? (
+                                                    <VisibilityOffIcon style={{width: '14px', height: '14px'}} /> 
+                                                ) : (
+                                                    <VisibilityIcon style={{width: '14px', height: '14px'}} />
+                                                )}
+                                            </IconButton>
+                                        </InputAdornment>
+                                    </ExtraInfo>
+                                )
+                            }
+                        />
+                        {this.props.helperText && 
+                            <FormHelperText style={{fontSize: '10px', lineHeight: '10px', margin: '4px 6px', whiteSpace: 'nowrap'}}>
+                                {this.props.helperText}
+                            </FormHelperText>
                         }
-                        onChange={this.handleChange}
-                        onKeyDown={(event: React.KeyboardEvent) => { if ((!this.props.multiline && event.key == 'Enter') || event.key == 'Escape') this.textField.current.blur() }}
-                        onFocus={this.handleFocus}
-                        onBlur={this.handleBlur}
-                        type={this.props.password && this.state.hidePassword ? 'password' : 'text'}
-                        multiline={this.props.multiline}
-                        required={this.props.required}
-                        rows={(() => {
-                            // Minimum show 5 lines
-                            const lines = this.state.textValue.split('\n').length;
-                            if (lines < 5) return 5;
-                            return lines;
-                        })()}
-                        endAdornment={
-                            !(this.props.disabledMessage == undefined || this.props.disabledMessage == '') ? (
-                                <InputAdornment position="end" title={this.props.disabledMessage} style={{height: '20px', margin: '0px 3px 0px 0px'}}>
-                                    <IconButton
-                                        disabled
-                                        style={{padding: '3px 3px 3px 0px'}}
-                                    >
-                                        <LockIcon style={{width: '14px', height: '14px'}} />
-                                    </IconButton>
-                                </InputAdornment>
-                            ) : this.props.password && (
-                                <InputAdornment position="end" title={this.state.hidePassword ? 'Show Password' : 'Hide Password'} style={{height: '20px', margin: '0px 3px 0px 0px'}}>
-                                    <IconButton
-                                        onClick={() => this.setState({hidePassword: !this.state.hidePassword})}
-                                        style={{padding: '3px 3px 3px 0px'}}
-                                    >
-                                        {this.state.hidePassword ? (
-                                            <VisibilityOffIcon style={{width: '14px', height: '14px'}} /> 
-                                        ) : (
-                                            <VisibilityIcon style={{width: '14px', height: '14px'}} />
-                                        )}
-                                    </IconButton>
-                                </InputAdornment>
-                            )
-                        }
-                    />
-                    {this.props.helperText && 
-                        <FormHelperText style={{fontSize: '10px', lineHeight: '10px', margin: '4px 6px', whiteSpace: 'nowrap'}}>
-                            {this.props.helperText}
-                        </FormHelperText>
-                    }
-                </FormControl>
+                    </FormControl>
+                {/* </ExtraInfo> */}
             </div>
         )
     }
 
+    public componentDidMount = () => {
+        this._isMounted = true
+    }
+
+    public componentWillUnmount = () => {
+        this._isMounted = false
+    }
+
+    private safeSetState = (map: any) => {
+		if (this._isMounted) {
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
+		}
+	}
+
     public shouldComponentUpdate = (nextProps: IProps<any>, nextState: IState<any>): boolean => {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
