@@ -8,6 +8,7 @@
 import * as React from 'react'
 import InputArea from './InputArea'
 import OutputArea from './OutputArea'
+import { Global } from '../../Global';
 
 interface IProps {
     cell: any
@@ -19,6 +20,7 @@ interface IState {}
 export default class CodeCell extends React.Component<IProps, IState> {
 
     public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         return (
             <div className='p-Widget lm-Widget jp-Cell jp-CodeCell jp-Notebook-cell' onContextMenu={(e) => e.preventDefault()}>
                 <InputArea cell={this.props.cell} metadata={this.props.metadata} />
