@@ -39,6 +39,7 @@ import WarningPopup from '../core/WarningPopup';
 import FileServerUtils from '../utils/FileServerUtils';
 import NotebookUtils from '../utils/NotebookUtils';
 import FormatUtils from '../utils/FormatUtils';
+import ExtraInfo from '../utils/ExtraInfo';
 
 export enum Phase {
 	Initializing = 'initializing',
@@ -140,17 +141,22 @@ export class App {
 				if (this._notebook.cells[0].execution_count == null) {
 					this._notebook.cells[0].execution_count = '*';
 				}
-				this._notebook.cells[0].outputs.push({
-					name: 'stdout',
-					output_type: 'stream',
-					text: [line],
-				});
+				var outputs = this._notebook.cells[0].outputs;
+				if (outputs.length == 0) {
+					outputs.push({
+						name: 'stdout',
+						output_type: 'stream',
+						text: this.formatForNotebook([line]),
+					});
+				} else {
+					const text: string[] = outputs[outputs.length-1].text;
+					text.push(line);
+					outputs[outputs.length-1].text = this.formatForNotebook(text)
+				}
 			}
 		}
 	}
 
-	private readonly _script: string;
-
 	private _name: string;
 	private _uuid: string = "";
 	private _modules: Module[] = [];
@@ -162,10 +168,7 @@ export class App {
 
 	private _timestamp: Date;
 
-	// Polling timer
-	private _polling: boolean;
-
-	constructor(name: string, notebook: any, script: string, uuid: string = "",
+	constructor(name: string, notebook: any, uuid: string = "",
 		initializing: Update[] = [], uploading: Update[] = [], requisitioning: Update[] = [], running: Update[] = [], timestamp = new Date()) {		
 
 		this._notebook = notebook;
@@ -173,7 +176,6 @@ export class App {
 		const optumi: OptumiMetadata = new OptumiMetadata(this._notebook["metadata"]["optumi"] || {});
 		this._interactive = optumi.interactive;
 
-		this._script = script;
 		this._name = name;
 		this._uuid = uuid;
 
@@ -191,8 +193,6 @@ export class App {
 
 		this._timestamp = timestamp;
 
-		this._polling = false;
-
 		// Handle errors where we were unable to load some of the updates
 		if (this._running.started) {
 			if (!this._requisitioning.completed) {
@@ -266,7 +266,7 @@ export class App {
         for (let i = 0; i < appMap.running.length; i++) {
             running.push(new Update(appMap.running[i], appMap.runningmod[i]));
         }
-		var app: App = new App(appMap.name, JSON.parse(appMap.notebook), appMap.script, appMap.uuid, initializing, uploading, requisitioning, running, new Date(appMap.timestamp));
+		var app: App = new App(appMap.name, JSON.parse(appMap.notebook), appMap.uuid, initializing, uploading, requisitioning, running, new Date(appMap.timestamp));
 		// Add modules
 		for (let module of appMap.modules) {
             const output: Update[] = [];
@@ -285,18 +285,70 @@ export class App {
                 // The module is still running
                 if (!app._initializing.completed || !app._uploading.completed || !app._requisitioning.completed || !app._running.completed) {
                     // Don't poll the module if the app is not running
-					mod.startPolling(app.interactive);
+					if (app.interactive) mod.startSessionHandler();
                 }
 			}
 		}
-		if (!app._initializing.completed || !app._uploading.completed || !app._requisitioning.completed || !app._running.completed) {
-			app.startPolling();
-		}
 		return app;
 	}
 
-	get script(): string {
-		return this._script;
+	public handleUpdate(body: any) {
+		let updated = false
+		if (body.initializing != null) {
+			if (body.initializing.length > 0) updated = true;
+			for (let i = 0; i < body.initializing.length; i++) {
+				this._initializing.addUpdate(new Update(body.initializing[i], body.initializingmod[i]), false);
+			}
+		}
+		if (body.uploading != null) {
+			if (body.uploading.length > 0) updated = true;
+			for (let i = 0; i < body.uploading.length; i++) {
+				this._uploading.addUpdate(new Update(body.uploading[i], body.uploadingmod[i]), false);
+			}
+		}
+		if (body.requisitioning != null) {
+			if (body.requisitioning.length > 0) updated = true;
+			for (let i = 0; i < body.requisitioning.length; i++) {
+				this._requisitioning.addUpdate(new Update(body.requisitioning[i], body.requisitioningmod[i]), false);
+				// Special case a warning when we fail to get a machine and start trying a new one
+				if (body.requisitioning[i] == 'Machine unavailable, trying another') {
+					if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+					Global.snackbarChange.emit(new Snackbar(
+						body.requisitioning[i],
+						{ variant: 'warning', }
+					));
+				}
+				// Special case loading bar while waiting for a server
+				if (this._requisitioning.message == "Waiting for cloud provider...") {
+					this._requisitioning.total = -1;
+				} else {
+					this._requisitioning.total = 0;
+				}
+			}
+		}
+		if (body.running != null) {
+			if (body.running.length > 0) updated = true;
+			for (let i = 0; i < body.running.length; i++) {
+				if (body.running[i] == 'Completed') {
+					if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+					Global.snackbarChange.emit(new Snackbar(
+						body.running[i],
+						{ variant: 'success', }
+					));
+				} else if (body.running[i] == 'Failed') {
+					if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+					Global.snackbarChange.emit(new Snackbar(
+						body.running[i],
+						{ variant: 'error', }
+					));
+				}
+				this._running.addUpdate(new Update(body.running[i], body.runningmod[i]), false);
+			}
+		}
+		if (updated) {
+			if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+			this._changed.emit(this);
+		}
 	}
 
 	get notebook(): any {
@@ -368,7 +420,10 @@ export class App {
 	}
 
 	public addModule(mod: Module) {
-		mod.changed.connect(() => this._changed.emit(this), this);
+		mod.changed.connect(() => {
+			if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+			return this._changed.emit(this)
+		}, this);
 		this._modules.push(mod);
 	}
 	
@@ -428,7 +483,6 @@ export class App {
 					path: this._name,
 					content: JSON.stringify(this._notebook),
 				},
-				script: this._script,
 			}),
 		};
 		return ServerConnection.makeRequest(
@@ -439,6 +493,7 @@ export class App {
 			Global.handleResponse(response);
 			return response.json();
 		}).then((body: any) => {
+			if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
 			Global.jobLaunched.emit(void 0);
 			this._uuid = body.uuid;
 			this._initializing.appUUID = this._uuid;
@@ -447,15 +502,21 @@ export class App {
 			this._running.appUUID = this._uuid;
 			this._initializing.addUpdate(new Update("Initializing...", ""));
 			appTracker.addApp(this);
+			if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
 			this._changed.emit(this);
 			this.launchNotebook();
 		});
 	}
 
+	private previousLaunchStatus: any;
+	private pollingDelay = 500;
 	private getLaunchStatus() {
         // If there is an unsigned agreement, do not poll
         if (Global.user != null && Global.user.unsignedAgreement) {
-            if (!this.failed) setTimeout(() => this.getLaunchStatus(), 500);
+            if (!this.failed) {
+				if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+				setTimeout(() => this.getLaunchStatus(), this.pollingDelay);
+			}
             return;
         }
 		const settings = ServerConnection.makeSettings();
@@ -473,7 +534,10 @@ export class App {
 		).then((response: Response) => {
 			Global.handleResponse(response);
 			if (response.status == 204) {
-				if (!this.failed) setTimeout(() => this.getLaunchStatus(), 500);
+				if (!this.failed) {
+					if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+					setTimeout(() => this.getLaunchStatus(), this.pollingDelay);
+				}
 				return;
 			}
 			return response.json();
@@ -483,7 +547,7 @@ export class App {
 					for (let i = 0; i < body.modules.length; i++) {
 						const mod = new Module(body.modules[i], this._uuid, this.addOutput);
 						this.addModule(mod);
-						mod.startPolling(this.interactive);
+						if (this.interactive) mod.startSessionHandler();
 					}
 				} else if (body.status == "Failed") {
 					if (!this._initializing.completed) {
@@ -496,25 +560,40 @@ export class App {
 						this._uploading.addUpdate(new Update("stop", ""));
 					}
 					if (body.snackbar) {
+						if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
 						Global.snackbarChange.emit(new Snackbar(
                             body.snackbar,
                             { variant: 'error', }
                         ));
 					}
 				} else {
-					if (!this.failed) setTimeout(() => this.getLaunchStatus(), 500);
+					if (!this.failed) {
+						if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+						setTimeout(() => this.getLaunchStatus(), this.pollingDelay);
+					}
                 }
-                this._changed.emit(this);
+				if (JSON.stringify(body) !== JSON.stringify(this.previousLaunchStatus)) {
+					if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+					this._changed.emit(this);
+					this.previousLaunchStatus = body
+				}
             }
 		}, (error: ServerConnection.ResponseError) => {
-			if (!this.failed) setTimeout(() => this.getLaunchStatus(), 500);
+			if (!this.failed) {
+				if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+				setTimeout(() => this.getLaunchStatus(), this.pollingDelay);
+			}
 		});
 	}
 
+	private previousCompressionProgress: any
 	private getCompressionProgress() {
         // If there is an unsigned agreement, do not poll
         if (Global.user != null && Global.user.unsignedAgreement) {
-            if (!this.failed) setTimeout(() => this.getCompressionProgress(), 500);
+            if (!this.failed) {
+				if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+				setTimeout(() => this.getCompressionProgress(), this.pollingDelay);
+			}
             return;
         }
 		const settings = ServerConnection.makeSettings();
@@ -532,7 +611,10 @@ export class App {
 		).then((response: Response) => {
 			Global.handleResponse(response);
 			if (response.status == 204) {
-				if (!this.failed) setTimeout(() => this.getCompressionProgress(), 500);
+				if (!this.failed) {
+					if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+					setTimeout(() => this.getCompressionProgress(), this.pollingDelay);
+				}
 				return;
 			}
 			return response.json();
@@ -546,19 +628,33 @@ export class App {
 				if (body.read != 0 && body.read == body.total) {
                     // Do nothing
                 } else {
-					if (!this.failed) setTimeout(() => this.getCompressionProgress(), 500);
+					if (!this.failed) {
+						if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+						setTimeout(() => this.getCompressionProgress(), this.pollingDelay);
+					}
+				}
+				if (JSON.stringify(body) !== JSON.stringify(this.previousCompressionProgress)) {
+					if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+					this._changed.emit(this);
+					this.previousCompressionProgress = body
 				}
-				this._changed.emit(this);
 			}
 		}, (error: ServerConnection.ResponseError) => {
-			if (!this.failed) setTimeout(() => this.getLaunchStatus(), 500);
+			if (!this.failed) {
+				if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+				setTimeout(() => this.getCompressionProgress(), this.pollingDelay);
+			}
 		});
 	}
 
+	private previousUploadProgress: any
 	private getUploadProgress() {
         // If there is an unsigned agreement, do not poll
         if (Global.user != null && Global.user.unsignedAgreement) {
-            if (!this.failed) setTimeout(() => this.getUploadProgress(), 500);
+            if (!this.failed) {
+				if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+				setTimeout(() => this.getUploadProgress(), this.pollingDelay);
+			}
             return;
         }
 		const settings = ServerConnection.makeSettings();
@@ -576,7 +672,10 @@ export class App {
 		).then((response: Response) => {
 			Global.handleResponse(response);
 			if (response.status == 204) {
-				if (!this.failed) setTimeout(() => this.getUploadProgress(), 500);
+				if (!this.failed) {
+					if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+					setTimeout(() => this.getUploadProgress(), this.pollingDelay);
+				}
 				return;
 			}
 			return response.json();
@@ -591,14 +690,23 @@ export class App {
 				this._uploading.total = body.total;
 				if (body.read != 0 && body.read == body.total) {
 					this._uploading.addUpdate(new Update("stop", ""));
-					this.startPolling();
 				} else {
-					if (!this.failed) setTimeout(() => this.getUploadProgress(), 500);
+					if (!this.failed) {
+						if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+						setTimeout(() => this.getUploadProgress(), this.pollingDelay);
+					}
+				}
+				if (JSON.stringify(body) !== JSON.stringify(this.previousUploadProgress)) {
+					if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+					this._changed.emit(this);
+					this.previousUploadProgress = body
 				}
-				this._changed.emit(this);
 			}
 		}, (error: ServerConnection.ResponseError) => {
-			if (!this.failed) setTimeout(() => this.getLaunchStatus(), 500);
+			if (!this.failed) {
+				if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
+				setTimeout(() => this.getUploadProgress(), this.pollingDelay);
+			}
 		});
 	}
 
@@ -606,13 +714,13 @@ export class App {
 	private async launchNotebook() {
 		const optumi: OptumiMetadata = new OptumiMetadata(this._notebook["metadata"]["optumi"] || {});
 		const uploadFiles: FileUploadMetadata[] = optumi.upload.files;
-		const requirementsFileName: string = optumi.upload.requirements;
+		const requirements: string = optumi.upload.requirements;
 		const compressFiles = Global.user.compressFilesEnabled;
 
 		var data: any = {};
 
-		if (requirementsFileName != null) {
-			data.requirementsFile =  requirementsFileName;
+		if (requirements != null) {
+			data.requirementsFile =  requirements;
 		}
 
 		data.dataFiles = [];
@@ -748,99 +856,6 @@ export class App {
 		return false;
 	}
 
-	private receiveUpdate() {
-		// If there is an unsigned agreement, do not poll
-        if (Global.user != null && Global.user.unsignedAgreement) {
-            setTimeout(() => this.receiveUpdate(), 500);
-            return;
-        }
-		const settings = ServerConnection.makeSettings();
-		const url = settings.baseUrl + "optumi/pull-workload-status-update";
-		const init: RequestInit = {
-			method: 'POST',
-			body: JSON.stringify({
-				uuid: this._uuid,
-				lastInitializingLine: this._initializing.length.toString(),
-				lastUploadingLine: this._uploading.length.toString(),
-				lastRequisitioningLine: this._requisitioning.length.toString(),
-				lastRunningLine: this._running.length.toString(),
-			}),
-		};
-		ServerConnection.makeRequest(
-			url,
-			init, 
-			settings
-		).then((response: Response) => {
-			if (this._polling) {
-				// If we are polling, send a new request in 2 seconds
-				setTimeout(() => this.receiveUpdate(), 2000);
-			}
-			Global.handleResponse(response);
-			return response.json();
-		}).then((body: any) => {
-			if (body.initializing != null) {
-				for (let i = 0; i < body.initializing.length; i++) {
-					this._initializing.addUpdate(new Update(body.initializing[i], body.initializingmod[i]), false);
-				}
-			}
-			if (body.uploading != null) {
-				for (let i = 0; i < body.uploading.length; i++) {
-					this._uploading.addUpdate(new Update(body.uploading[i], body.uploadingmod[i]), false);
-				}
-			}
-			if (body.requisitioning != null) {
-				for (let i = 0; i < body.requisitioning.length; i++) {
-					this._requisitioning.addUpdate(new Update(body.requisitioning[i], body.requisitioningmod[i]), false);
-                    // Special case a warning when we fail to get a machine and start trying a new one
-                    if (body.requisitioning[i] == 'Machine unavailable, trying another') {
-                        Global.snackbarChange.emit(new Snackbar(
-                            body.requisitioning[i],
-                            { variant: 'warning', }
-                        ));
-                    }
-                    // Special case loading bar while waiting for a server
-					if (this._requisitioning.message == "Waiting for cloud provider...") {
-						this._requisitioning.total = -1;
-					} else {
-						this._requisitioning.total = 0;
-					}
-				}
-			}
-			if (body.running != null) {
-				for (let i = 0; i < body.running.length; i++) {
-                    if (body.running[i] == 'Completed') {
-                        Global.snackbarChange.emit(new Snackbar(
-                            body.running[i],
-                            { variant: 'success', }
-                        ));
-                    } else if (body.running[i] == 'Failed') {
-                        Global.snackbarChange.emit(new Snackbar(
-                            body.running[i],
-                            { variant: 'error', }
-                        ));
-                    }
-					this._running.addUpdate(new Update(body.running[i], body.runningmod[i]), false);
-				}
-			}
-			if (this.getAppStatus() == Status.Completed) {
-				this.stopPolling();
-			}
-			this._changed.emit(this);
-		});
-	}
-
-	public startPolling() {
-		this._polling = true;
-		this.receiveUpdate();
-	}
-
-	public stopPolling() {
-        this._polling = false;
-        for (let module of this._modules) {
-            module.stopPolling(this.interactive);
-        }
-	}
-
 	public getComponent(): React.CElement<IProps, AppComponent> {
 		return React.createElement(AppComponent, {key: this._uuid, app: this});
 	}
@@ -859,14 +874,16 @@ export class App {
 	public getIdentityComponent() {
 		var app: App = this
 		return (
-			<div style={{paddingLeft: '3px'}} title={app.name}> {/* the padding is 7px because the height it needs to take up is 36 and the height of this 16px font is 22px */}
-				<div style={{paddingBottom: '3px', fontSize: '13px', lineHeight: '1', fontWeight: 'normal', whiteSpace: 'nowrap', overflow: 'hidden', textOverflow: 'ellipsis'}}>
-					{app.name.split('/').pop()}
+			<ExtraInfo reminder={app.name}>
+				<div style={{paddingLeft: '3px'}}> {/* the padding is 7px because the height it needs to take up is 36 and the height of this 16px font is 22px */}
+					<div style={{paddingBottom: '3px', fontSize: '13px', lineHeight: '1', fontWeight: 'normal', whiteSpace: 'nowrap', overflow: 'hidden', textOverflow: 'ellipsis'}}>
+						{app.name.split('/').pop()}
+					</div>
+					<span style={{fontSize: '13px', lineHeight: '1', fontWeight: 'normal', color: 'gray'}}>
+						{this.formatTime()}
+					</span>
 				</div>
-				<span style={{fontSize: '13px', lineHeight: '1', fontWeight: 'normal', color: 'gray'}}>
-					{this.formatTime()}
-				</span>
-			</div>
+			</ExtraInfo>
 		)
 	}
 }
@@ -904,8 +921,8 @@ class AppComponent extends React.Component<IProps, IState> {
 	}
 
 	private handleDeleteClicked = () => {
-		this.checkAndSetState({ waiting: true, spinning: false });
-		setTimeout(() => this.checkAndSetState({ spinning: true }), 1000);
+		this.safeSetState({ waiting: true, spinning: false });
+		setTimeout(() => this.safeSetState({ spinning: true }), 1000);
 		const settings = ServerConnection.makeSettings();
 		const url = settings.baseUrl + "optumi/teardown-notebook";
 		const init: RequestInit = {
@@ -919,7 +936,7 @@ class AppComponent extends React.Component<IProps, IState> {
 			init,
 			settings
 		).then((response: Response) => {
-			this.checkAndSetState({ waiting: false });
+			this.safeSetState({ waiting: false });
 			Global.handleResponse(response);
 			Global.user.appTracker.removeApp(this.props.app.uuid);
 		});
@@ -934,8 +951,8 @@ class AppComponent extends React.Component<IProps, IState> {
 	}
 
     private handleStopClicked = () => {
-		this.checkAndSetState({ waiting: true, spinning: false });
-		setTimeout(() => this.checkAndSetState({ spinning: true }), 1000);
+		this.safeSetState({ waiting: true, spinning: false });
+		setTimeout(() => this.safeSetState({ spinning: true }), 1000);
 		const settings = ServerConnection.makeSettings();
 		const url = settings.baseUrl + "optumi/stop-notebook";
 		const init: RequestInit = {
@@ -949,19 +966,19 @@ class AppComponent extends React.Component<IProps, IState> {
 			init,
 			settings
 		).then((response: Response) => {
-			this.checkAndSetState({ waiting: false })
+			this.safeSetState({ waiting: false })
 			Global.handleResponse(response);
 		});
 	}
 
 	private getStatusColor = (): StatusColor => {
-		const appStatus = this.props.app.getAppStatus();
-		if (this.props.app.interactive && this.props.app.getAppMessage() == 'Closed') {
+		if (this.props.app.getAppMessage() == 'Closed' || this.props.app.getAppMessage() == 'Terminated') {
 			return StatusColor.DARK_GRAY;
 		}
 		if (this.props.app.getError()) {
 			return StatusColor.RED;
 		} else {
+			const appStatus = this.props.app.getAppStatus();
 			if (appStatus == Status.Initializing) {
 				return StatusColor.BLUE;
 			} else {
@@ -971,38 +988,68 @@ class AppComponent extends React.Component<IProps, IState> {
 	}
 
 	public render = (): JSX.Element => {
+		if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
 		var tags: JSX.Element[] = []
 		var app: App = this.props.app
 
 		// Get the right progress message...
 		var appMessage = app.getAppMessage();
+		const loadingTooltip = this.props.app.getLoadingTooltip()
 		tags.push(
-			<Tag id={app.uuid + appMessage} key={'appMessage' + appMessage}
-				icon={app.getAppStatus() == Status.Completed ? (app.getError() ? (
-					(app.getAppMessage() == 'Closed' ? (
-						<ClearIcon style={{height: '14px', width: '14px', fill: 'gray'}} />
+			<ExtraInfo key={'appMessage'} reminder={(loadingTooltip === '' || loadingTooltip === undefined) ? 'Status' : loadingTooltip}>
+				<Tag key={'appMessage'}
+					id={app.uuid + appMessage}
+					icon={app.getAppStatus() == Status.Completed ? (app.getError() ? (
+						(app.getAppMessage() == 'Closed' || app.getAppMessage() == 'Terminated' ? (
+							<ClearIcon style={{
+								height: '14px',
+								width: '14px',
+								fill: 'gray',
+							}} />
+						) : (
+							<ClearIcon style={{
+								height: '14px',
+								width: '14px',
+								fill: '#f48f8d',
+							}} />
+						))
 					) : (
-						<ClearIcon style={{height: '14px', width: '14px', fill: '#f48f8d'}} />
-					))
-				) : (
-					<CheckIcon style={{height: '14px', width: '14px', fill: '#68da7c'}} />
-				)) : undefined}
-				label={appMessage}
-				color={(this.props.app.getShowLoading() || app.getAppStatus() == Status.Completed) ? this.getStatusColor() : undefined}
-				showLoading={this.props.app.getShowLoading()}
-				percentLoaded={this.props.app.getPercentLoaded()}
-				tooltip={this.props.app.getLoadingTooltip()}
-			/>
+						<CheckIcon style={{
+							height: '14px',
+							width: '14px',
+							fill: '#68da7c',
+						}} />
+					)) : undefined}
+					label={appMessage}
+					color={(this.props.app.getShowLoading() || app.getAppStatus() == Status.Completed) ? this.getStatusColor() : undefined}
+					showLoading={this.props.app.getShowLoading()}
+					percentLoaded={this.props.app.getPercentLoaded()}
+				/>
+			</ExtraInfo>
 		)
         var appElapsed = app.getTimeElapsed();
-		tags.push(<Tag key={'appElapsed' + appElapsed} label={appElapsed} />)
+		tags.push(
+			<ExtraInfo key={'appElapsed'} reminder='Duration'>
+				<Tag key={'appElapsed'} label={appElapsed} />
+			</ExtraInfo>
+		)
 		var appCost = app.getCost();
-		if (appCost) tags.push(<Tag key={'appCost' + appCost} label={appCost} />)
+		if (appCost) {
+			tags.push(
+				<ExtraInfo key={'appCost'} reminder='Cost'>
+					<Tag key={'appCost'} label={appCost} />
+				</ExtraInfo>
+			)
+		}
         return (
 			<>
                 <StatusWrapper key={this.props.app.uuid} statusColor={app.getAppStatus() == Status.Completed ? 'var(--jp-layout-color2)' : this.getStatusColor()}>
                     <InfoSkirt
-                        leftButton={<DetailsDialog app={this.props.app} />}
+                        leftButton={
+							<ExtraInfo reminder='See details'>
+								<DetailsDialog app={this.props.app} />
+							</ExtraInfo>
+						}
                         rightButton={(this.props.app.requisitioning.completed && !this.props.app.requisitioning.error) && !this.props.app.running.completed ? (
 							<>
 								<WarningPopup
@@ -1020,7 +1067,7 @@ class AppComponent extends React.Component<IProps, IState> {
 										text: `Cancel`,
 										onCancel: (prevent: boolean) => {
 											// this.saveStopJobPreventValue(prevent)
-											this.checkAndSetState({ showStopJobPopup: false })
+											this.safeSetState({ showStopJobPopup: false })
 										},
 									}}
 									continue={{
@@ -1032,7 +1079,7 @@ class AppComponent extends React.Component<IProps, IState> {
 											}
 										})(),
 										onContinue: (prevent: boolean) => {
-											this.checkAndSetState({ showStopJobPopup: false })
+											this.safeSetState({ showStopJobPopup: false })
 											this.saveStopJobPreventValue(prevent)
 											this.handleStopClicked()
 										},
@@ -1045,12 +1092,25 @@ class AppComponent extends React.Component<IProps, IState> {
 										if (this.getStopJobPreventValue()) {
 											this.handleStopClicked()
 										} else {
-											this.checkAndSetState({ showStopJobPopup: true })
+											this.safeSetState({ showStopJobPopup: true })
 										}
 									}}
-									style={{position: 'relative', display: 'inline-block', width: '36px', height: '36px', padding: '3px'}}
+									style={{
+										position: 'relative',
+										display: 'inline-block',
+										width: '36px',
+										height: '36px',
+										padding: '3px',
+									}}
 								>
-									<StopIcon style={{position: 'relative', width: '30px', height: '30px', padding: '3px'}} />
+									<ExtraInfo reminder={this.props.app.interactive ? 'Stop' : 'Terminate'}>
+										<StopIcon style={{
+											position: 'relative',
+											width: '30px',
+											height: '30px',
+											padding: '3px',
+										}} />
+									</ExtraInfo>
 									{this.state.waiting && this.state.spinning && <CircularProgress size='30px' style={{position: 'absolute'}}/>}
 								</IconButton>
 							</>
@@ -1071,33 +1131,35 @@ class AppComponent extends React.Component<IProps, IState> {
 										text: `Cancel`,
 										onCancel: (prevent: boolean) => {
 											// this.saveDeleteJobPreventValue(prevent)
-											this.checkAndSetState({ showDeleteJobPopup: false })
+											this.safeSetState({ showDeleteJobPopup: false })
 										},
 									}}
 									continue={{
 										text: `Delete it`,
 										onContinue: (prevent: boolean) => {
-											this.checkAndSetState({ showDeleteJobPopup: false })
+											this.safeSetState({ showDeleteJobPopup: false })
 											this.saveDeleteJobPreventValue(prevent)
 											this.handleDeleteClicked()
 										},
 										color: `error`,
 									}}
 								/>
-								<IconButton
-									disabled={this.state.waiting || !this.props.app.initializing.completed}
-									onClick={() => {
-										if (this.getDeleteJobPreventValue()) {
-											this.handleDeleteClicked()
-										} else {
-											this.checkAndSetState({ showDeleteJobPopup: true })
-										}
-									}}
-									style={{position: 'relative', display: 'inline-block', width: '36px', height: '36px', padding: '3px'}}
-								>
-									<DeleteIcon style={{position: 'relative', width: '30px', height: '30px', padding: '3px'}} />
-									{this.state.waiting && this.state.spinning && <CircularProgress size='30px' style={{position: 'absolute'}}/>}
-								</IconButton>
+								<ExtraInfo reminder='Delete'>
+									<IconButton
+										disabled={this.state.waiting || !this.props.app.initializing.completed}
+										onClick={() => {
+											if (this.getDeleteJobPreventValue() || !this.props.app.running.started) {
+												this.handleDeleteClicked()
+											} else {
+												this.safeSetState({ showDeleteJobPopup: true })
+											}
+										}}
+										style={{position: 'relative', display: 'inline-block', width: '36px', height: '36px', padding: '3px'}}
+									>
+										<DeleteIcon style={{position: 'relative', width: '30px', height: '30px', padding: '3px'}} />
+										{this.state.waiting && this.state.spinning && <CircularProgress size='30px' style={{position: 'absolute'}}/>}
+									</IconButton>
+								</ExtraInfo>
 							</>
 						)}
                         tags={tags}
@@ -1113,26 +1175,42 @@ class AppComponent extends React.Component<IProps, IState> {
         try {
             if (JSON.stringify(this.props) != JSON.stringify(nextProps)) return true;
             if (JSON.stringify(this.state) != JSON.stringify(nextState)) return true;
-            if (Global.shouldLogOnRender) console.log('SuppressedRender')
+            if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
             return false;
         } catch (error) {
             return true;
         }
     }
 
-	private checkAndSetState = (map: any) => {
+	private safeSetState = (map: any) => {
 		if (this._isMounted) {
-			this.setState(map);
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
 		}
 	}
 
 	// Will be called automatically when the component is mounted
-	componentDidMount = () => {
+	public componentDidMount = () => {
         this._isMounted = true;
 	}
 
 	// Will be called automatically when the component is unmounted
-	componentWillUnmount = () => {
+	public componentWillUnmount = () => {
 		this._isMounted = false;
 	}
 }
