@@ -46,9 +46,6 @@ export class Module {
     private _files: OutputFile[];
     private _updates: string[];
 
-	// Polling timer
-	private _polling: boolean;
-
 	constructor(uuid: string, workloadUUID: string, addOutput: (line: string, modifier: string) => void, machine: Machine = null, sessionToken: string = null, output: Update[] = [], updates: string[] = [], files: OutputFile[] = []) {
 		this._uuid = uuid;
 		this._workloadUUID = workloadUUID;
@@ -66,8 +63,6 @@ export class Module {
 
 		this._files = files;
 		this._updates = updates;
-
-		this._polling = false;
 	}
 
 	get uuid(): string {
@@ -90,6 +85,10 @@ export class Module {
         return this._machine;
     }
 
+	get updates(): String[] {
+        return this._updates;
+    }
+
     get output(): Update[] {
         return this._output;
     }
@@ -106,72 +105,53 @@ export class Module {
 		return this._updates.includes("error");
     }
 
-	private async receiveUpdate() {
-        // If there is an unsigned agreement, do not poll
-        if (Global.user != null && Global.user.unsignedAgreement) {
-            if (this._polling) setTimeout(() => this.receiveUpdate(), 500);
-            return;
-        }
-		const settings = ServerConnection.makeSettings();
-		const url = settings.baseUrl + "optumi/pull-module-status-update";
-		const init: RequestInit = {
-			method: 'POST',
-			body: JSON.stringify({
-				workloadUUID: this._workloadUUID,
-				moduleUUID: this._uuid,
-				lastUpdateLine: this._updates.length.toString(),
-				lastOutputLine: this._output.length.toString(),
-			}),
-		};
-		ServerConnection.makeRequest(
-			url,
-			init, 
-			settings
-		).then((response: Response) => {
-			if (this._polling) {
-				// If we are polling, send a new request in half a second
-				setTimeout(() => this.receiveUpdate(), 500);
-			}
-			Global.handleResponse(response);
-			return response.json();
-		}).then((body: any) => {
-			if (body.output != null) {
-				for (let i = 0; i < body.output.length; i++) {
-					this.addOutput(body.output[i], body.outputmod[i]);
-					this._output.push(new Update(body.output[i], body.outputmod[i]));
-					if (body.output[i].includes('Jupyter Server ') && body.output[i].includes(' is running at:')) {
-						this._sessionReady = true;
-						window.open('http://localhost:' + this._sessionPort + '?token=' + this._sessionToken, '_blank');
-					}
-				}
-            }
-            if (body.files != null) {
-                this._files = [];
-				for (let i = 0; i < body.files.length; i++) {
-                    if (body.files[i] != '') {
-                        this._files.push(new OutputFile(body.files[i], body.filesmod[i], body.filessize[i]));
-                    }
+	public handleUpdate(body: any) {
+		let updated = false
+		if (body.output != null) {
+			if (body.output.length > 0) updated = true;
+			for (let i = 0; i < body.output.length; i++) {
+				this.addOutput(body.output[i], body.outputmod[i]);
+				this._output.push(new Update(body.output[i], body.outputmod[i]));
+				if (body.output[i].includes('Jupyter Server ') && body.output[i].includes(' is running at:')) {
+					this._sessionReady = true;
+					window.open('http://localhost:' + this._sessionPort + '?token=' + this._sessionToken, '_blank');
 				}
 			}
-			if (body.token != null) {
-				this._sessionToken = body.token;
-			}
-			if (body.updates != null) {
-				for (var update of body.updates) {
-					this._updates.push(update);
-					if (update == "stop") {
-                        this._updates.push("stop");
-					} else if (update == "launched") {
-					} else if (update == "closed") {
-					}
+		}
+		if (body.files != null) {
+			this._files = [];
+			for (let i = 0; i < body.files.length; i++) {
+				if (body.files[i] != '') {
+					updated = true
+					this._files.push(new OutputFile(body.files[i], body.filesmod[i], body.filessize[i]));
 				}
-            }
-            if (body.machine != null) {
-                this._machine = Object.setPrototypeOf(body.machine, Machine.prototype);
-            }
+			}
+		}
+		if (body.token != null) {
+			updated = true
+			this._sessionToken = body.token;
+		}
+		if (body.updates != null) {
+			if (body.updates.length > 0) updated = true;
+			for (var update of body.updates) {
+				this._updates.push(update);
+				if (update == "stop") {
+					this._updates.push("stop");
+				} else if (update == "launched") {
+				} else if (update == "closed") {
+					this.stopSessionHandler();
+				}
+			}
+		}
+		if (body.machine != null) {
+			updated = true;
+			this._machine = Object.setPrototypeOf(body.machine, Machine.prototype);
+		}
+		if (updated) {
+			if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
 			this._changed.emit(this);
-		});
-    }
+		}
+	}
 
     public pushModuleInput = (line: string) => {
         const settings = ServerConnection.makeSettings();
@@ -193,56 +173,45 @@ export class Module {
 		});
     }
 
-	public startPolling(interactive = false) {
-		this._polling = true;
-		this.receiveUpdate();
-
-		// Start a session handler if necessary
-		if (interactive) {
-			const settings = ServerConnection.makeSettings();
-			const url = settings.baseUrl + "optumi/connect-session";
-			const init: RequestInit = {
-				method: 'POST',
-				body: JSON.stringify({
-					module: this._uuid,
-				}),
-			};
-			ServerConnection.makeRequest(
-				url,
-				init, 
-				settings
-			).then((response: Response) => {
-				Global.handleResponse(response);
-				return response.json();
-			}).then((body: any) => {
-				if (body.port) {
-					this._sessionPort = body.port;
-				}
-			});
-		}
+	public startSessionHandler() {
+		const settings = ServerConnection.makeSettings();
+		const url = settings.baseUrl + "optumi/connect-session";
+		const init: RequestInit = {
+			method: 'POST',
+			body: JSON.stringify({
+				module: this._uuid,
+			}),
+		};
+		ServerConnection.makeRequest(
+			url,
+			init, 
+			settings
+		).then((response: Response) => {
+			Global.handleResponse(response);
+			return response.json();
+		}).then((body: any) => {
+			if (body.port) {
+				this._sessionPort = body.port;
+			}
+		});
 	}
 
-	public stopPolling(interactive = false) {
-		this._polling = false;
-
-		// Stop a session handler if necessary
-		if (interactive) {
-			const settings = ServerConnection.makeSettings();
-			const url = settings.baseUrl + "optumi/disconnect-session";
-			const init: RequestInit = {
-				method: 'POST',
-				body: JSON.stringify({
-					module: this._uuid,
-				}),
-			};
-			ServerConnection.makeRequest(
-				url,
-				init, 
-				settings
-			).then((response: Response) => {
-				Global.handleResponse(response);
-				return response.text();
-			});
-		}
+	public stopSessionHandler() {
+		const settings = ServerConnection.makeSettings();
+		const url = settings.baseUrl + "optumi/disconnect-session";
+		const init: RequestInit = {
+			method: 'POST',
+			body: JSON.stringify({
+				module: this._uuid,
+			}),
+		};
+		ServerConnection.makeRequest(
+			url,
+			init, 
+			settings
+		).then((response: Response) => {
+			Global.handleResponse(response);
+			return response.text();
+		});
 	}
 }
