@@ -9,6 +9,7 @@ import { ISignal, Signal } from '@lumino/signaling';
 
 import { NotebookPanel, NotebookTracker } from '@jupyterlab/notebook';
 import { OptumiMetadata } from './OptumiMetadata';
+import { Global } from '../Global'
 
 export class OptumiMetadataTracker {
     private _optumiMetadata = new Map<string, TrackedOptumiMetadata>();
@@ -25,6 +26,7 @@ export class OptumiMetadataTracker {
 
 	private handleCurrentChanged = async (current: NotebookPanel) => {
         if (current == null) {
+            if (Global.shouldLogOnPoll) console.log('FunctionPoll (' + new Date().getSeconds() + ')');
             setTimeout(() => this.handleCurrentChanged(this._tracker.currentWidget), 250);
             return;
         }
@@ -37,7 +39,8 @@ export class OptumiMetadataTracker {
 		this._optumiMetadata.set(path, new TrackedOptumiMetadata(path, metadata));
         // Save the metadata to make sure all files have valid metadata
 		rawMetadata.set("optumi", JSON.parse(JSON.stringify(metadata)));
-		this._metadataChanged.emit(void 0);
+        if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+        this._metadataChanged.emit(void 0);
 	}
 
 	public getMetadata = (): TrackedOptumiMetadata => {
@@ -52,7 +55,8 @@ export class OptumiMetadataTracker {
 	public setMetadata = (optumi: TrackedOptumiMetadata) => {
 		const rawMetadata = this._tracker.find(x => x.context.path == optumi.path).model.metadata;
 		rawMetadata.set("optumi", JSON.parse(JSON.stringify(optumi.metadata)));
-		this._metadataChanged.emit(void 0);
+        if (Global.shouldLogOnEmit) console.log('SignalEmit (' + new Date().getSeconds() + ')');
+        this._metadataChanged.emit(void 0);
 	}
 
 	public getMetadataChanged = (): ISignal<this, void> => {
