@@ -16,6 +16,7 @@ import { ShadowedDivider } from '../../core';
 import CloseIcon from '@material-ui/icons/Close';
 import { MachineCapability } from './MachineCapabilities';
 import FormatUtils from '../../utils/FormatUtils';
+import { App } from '../App';
 
 const StyledDialog = withStyles({
     paper: {
@@ -38,6 +39,8 @@ interface IProps {
     machine: Machine
     onOpen?: () => void
 	onClose?: () => void
+    onMouseOver?: (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void
+    onMouseOut?: (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void
 }
 
 interface IState {
@@ -47,6 +50,7 @@ interface IState {
 
 // TODO:Beck The popup needs to be abstracted out, there is too much going on to reproduce it in more than one file
 export class PopupMachineComponent extends React.Component<IProps, IState> {
+    private _isMounted = false
 
     constructor(props: IProps) {
         super(props);
@@ -58,17 +62,33 @@ export class PopupMachineComponent extends React.Component<IProps, IState> {
     
     private handleClickOpen = () => {
         if (this.props.onOpen) this.props.onOpen()
-		this.setState({ open: true });
+		this.safeSetState({ open: true });
 	}
 
 	private handleClose = () => {
-        this.setState({ open: false });
+        this.safeSetState({ open: false });
         if (this.props.onClose) this.props.onClose()
     }
 
+    private handleMouseOver = (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => {
+        if (this.props.onMouseOver) this.props.onMouseOver(event);
+    }
+
+    private handleMouseOut = (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => {
+        if (this.props.onMouseOut) this.props.onMouseOut(event);
+    }
+
     public render = (): JSX.Element => {
-        if (Global.shouldLogOnRender) console.log('PopupMachineRender (' + new Date().getSeconds() + ')');
+        if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
         const machine = this.props.machine
+        var placedApp: App;
+        if (machine.app) {
+            for (var app of Global.user.appTracker.activeJobsOrSessions) {
+                if (app.uuid == machine.app) {
+                    placedApp = app;
+                }
+            }
+        }
         return (
             <>
                 <IconButton
@@ -80,6 +100,8 @@ export class PopupMachineComponent extends React.Component<IProps, IState> {
                         height: '36px',
                         padding: '3px',
                     }}
+                    onMouseOver={this.handleMouseOver}
+                    onMouseOut={this.handleMouseOut}
                 >
                     <PopupIcon
                         style={{
@@ -149,7 +171,7 @@ export class PopupMachineComponent extends React.Component<IProps, IState> {
                                 <div style={{padding: '6px'}}>
                                     <Tabs
                                         value={this.state.selectedPanel}
-                                        onChange={(event, newValue) => this.setState({selectedPanel: newValue})}
+                                        onChange={(event, newValue) => this.safeSetState({selectedPanel: newValue})}
                                         orientation='vertical'
                                         variant='fullWidth'
                                         indicatorColor='primary'
@@ -190,12 +212,19 @@ export class PopupMachineComponent extends React.Component<IProps, IState> {
                                 {this.state.selectedPanel == Page.CAPABILITY ? (
                                     <MachineCapability machine={machine}/>
                                 ) : this.state.selectedPanel == Page.STATUS ? (
-                                    <div style={{ margin: '6px' }}>
-                                        {Machine.getStateMessage(this.props.machine.state) == '' ? 'Machine currently has no status' : 'Machine is currently ' + Machine.getStateMessage(this.props.machine.state).toLowerCase()}
-                                    </div>
+                                    <>
+                                        <div style={{ margin: '6px' }}>
+                                            {Machine.getStateMessage(this.props.machine.state) == '' ? 'Machine currently has no status' : 'Machine is currently ' + Machine.getStateMessage(this.props.machine.state).toLowerCase()}
+                                        </div>
+                                        {placedApp && (
+                                            <div style={{ margin: '6px' }}>
+                                                {(placedApp.interactive ? 'Session' : 'Job') + ' ' + placedApp.name + ' is ' + (((placedApp.requisitioning.completed && !placedApp.requisitioning.error) && !placedApp.running.completed) ? 'running on' : 'waiting for') + ' this machine.'}
+                                            </div>
+                                        )}
+                                    </>
                                 ) : this.state.selectedPanel == Page.COST && (
                                     <div style={{ margin: '6px' }}>
-                                        Machine costs {FormatUtils.styleRateUnitValue()(machine.rate)}
+                                        Machine costs {FormatUtils.styleRateUnitValue()(machine.rate) + (machine.promo ? ' (promotional price)' : '')}
                                     </div>
                                 )}
                             </DialogContent>
@@ -205,4 +234,34 @@ export class PopupMachineComponent extends React.Component<IProps, IState> {
             </>
         )
     }
+
+    public componentDidMount = () => {
+        this._isMounted = true
+    }
+
+    public componentWillUnmount = () => {
+        this._isMounted = false
+    }
+
+    private safeSetState = (map: any) => {
+		if (this._isMounted) {
+			let update = false
+			try {
+				for (const key of Object.keys(map)) {
+					if (JSON.stringify(map[key]) !== JSON.stringify((this.state as any)[key])) {
+						update = true
+						break
+					}
+				}
+			} catch (error) {
+				update = true
+			}
+			if (update) {
+				if (Global.shouldLogOnSafeSetState) console.log('SafeSetState (' + new Date().getSeconds() + ')');
+				this.setState(map)
+			} else {
+				if (Global.shouldLogOnSafeSetState) console.log('SuppressedSetState (' + new Date().getSeconds() + ')');
+			}
+		}
+	}
 }
\ No newline at end of file
