@@ -11,7 +11,7 @@ import * as React from 'react';
 import { Global } from '../Global';
 import { withStyles } from '@material-ui/core';
 
-const REMOVE_EXTRA_INFO_CAUSE_ISSUE = false
+const REMOVE_EXTRA_INFO_CAUSE_ISSUE = true
 
 // Include the bootstrap styles once
 if (!REMOVE_EXTRA_INFO_CAUSE_ISSUE) document.body.innerHTML += `<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.5.0/css/bootstrap.min.css" integrity="sha384-9aIt2nRpC12Uk9gS9baDl411NQApFmC26EwAOH8WgZl5MYYxFfc+NcPb1dKGj7Sk" crossorigin="anonymous" />`
@@ -26,7 +26,7 @@ interface ExtraInfoProps {
     other?: any,
 }
 
-export function ExtraInfo(props: ExtraInfoProps) {
+export default /*React.memo(*/function ExtraInfo(props: ExtraInfoProps) {
     const {
         // Props
         children,
@@ -50,10 +50,12 @@ export function ExtraInfo(props: ExtraInfoProps) {
 
     const componentDidMount = () => {
         Global.onInQuestionModeChange.connect(forceUpdate)
+        Global.themeManager.themeChanged.connect(forceUpdate)
     }
 
     const componentWillUnmount = () => {
         Global.onInQuestionModeChange.disconnect(forceUpdate)
+        Global.themeManager.themeChanged.disconnect(forceUpdate)
     }
 
     const handleMouseOver = (triggerHandler: any, ...args: any[]): any => {
@@ -76,30 +78,43 @@ export function ExtraInfo(props: ExtraInfoProps) {
             },
             '#styled-popover': {
                 zIndex: `1300`,
+                borderColor: `var(--jp-ui-font-color1)`,
             },
             '#styled-popover .popover-header': {
                 backgroundColor: `var(--jp-layout-color2)`,
-                fontWeight: Global.inQuestionMode && 'bold',
-                fontSize: !Global.inQuestionMode && '14px',
-                padding: !Global.inQuestionMode && '3px 6px',
+                borderRadius: '.3rem',
+                borderBottom: 'none',
+                fontWeight: Global.inQuestionMode ? 'bold' : undefined,
+                fontSize: !Global.inQuestionMode ? '14px' : undefined,
+                padding: !Global.inQuestionMode ? '3px 6px' : undefined,
             },
             '#styled-popover .popover-body': {
                 backgroundColor: `var(--jp-layout-color1)`,
             },
+            '#styled-popover .arrow::before': {
+                borderRightColor: Global.inQuestionMode ? `var(--jp-ui-font-color1)` : undefined,
+                borderTopColor: !Global.inQuestionMode ? `var(--jp-ui-font-color1)` : undefined,
+            },
             '#styled-popover .arrow::after': {
-                borderRightColor: Global.inQuestionMode && `var(--jp-layout-color1)`,
-                borderTopColor: !Global.inQuestionMode && `var(--jp-layout-color2)`,
+                borderRightColor: Global.inQuestionMode ? `var(--jp-layout-color1)` : undefined,
+                borderTopColor: !Global.inQuestionMode ? `var(--jp-layout-color2)` : undefined,
+                left: !Global.inQuestionMode ? undefined : (Global.themeManager == undefined || Global.themeManager.isLight(Global.themeManager.theme)) ? '1px' : '2px',
+                bottom: Global.inQuestionMode ? undefined : (Global.themeManager == undefined || Global.themeManager.isLight(Global.themeManager.theme)) ? '1px' : '2px',
             },
         }
     })(Popover)
 
-    return REMOVE_EXTRA_INFO_CAUSE_ISSUE ? children : ( // Render
+    const hasReminder = reminder !== undefined && reminder !== ''
+    const hasOverview = overview !== undefined && overview !== ''
+
+    if (Global.shouldLogOnRender) console.log('ComponentRender (' + new Date().getSeconds() + ')');
+    return (REMOVE_EXTRA_INFO_CAUSE_ISSUE || (!hasReminder && !hasOverview)) ? children : ( // Render
         <OverlayTrigger
             delay={{show: Global.inQuestionMode ? 10 : 500, hide: 10}}
             trigger={['hover', 'click', 'focus']}
             placement={Global.inQuestionMode ? 'right' : 'top'}
             overlay={Global.inQuestionMode ? (
-                <StyledPopover id='styled-popover'>
+                <StyledPopover /*key={'questionMode' + reminder + overview}*/ id='styled-popover'>
                     {reminder && (
                         <Popover.Title as='h3'>
                             {reminder}
@@ -112,7 +127,7 @@ export function ExtraInfo(props: ExtraInfoProps) {
                     )}
                 </StyledPopover>
             ) : (
-                <StyledPopover id='styled-popover'>
+                <StyledPopover /*key={reminder + overview}*/ id='styled-popover'>
                     {reminder && (
                         <Popover.Title as='h3'>
                             {reminder}
@@ -136,4 +151,12 @@ export function ExtraInfo(props: ExtraInfoProps) {
             }}
         </OverlayTrigger>
     )
-}
\ No newline at end of file
+}/*, (props: ExtraInfoProps, nextProps: ExtraInfoProps) => {
+    if (props.reminder !== nextProps.reminder) return false;
+    if (props.overview !== nextProps.overview) return false;
+    if (props.documentation !== nextProps.documentation) return false;
+    props.children.props
+    if (JSON.stringify(props.children) !== JSON.stringify(nextProps.children)) return false;
+    if (Global.shouldLogOnRender) console.log('SuppressedRender (' + new Date().getSeconds() + ')');
+    return true
+})*/
\ No newline at end of file
